#!/bin/bash

# Define services' names, classes, friendly names, lock file names and script names:
declare -a NAMES=('RMI' 'CONTROLLER' 'DEPLOYER' 'LOGSERVER' 'SCHEDULER');

declare -a CLASSES=('sun.rmi.registry.RegistryImpl' 'outsystems.hubedition.deploymentcontroller.Service' 'outsystems.hubedition.deployservice.Service' 
	'outsystems.hubedition.logservice.Service' 'outsystems.hubedition.scheduler.Service');

declare -a PRINTABLE_NAMES=('RMI Registry Service' 'OutSystems Deployment Controller Service' 'OutSystems Deployment Service' 'OutSystems Log Service' 
	'OutSystems Scheduler Service');

declare -a LOCK_NAMES=('.serviceRmiStarted' '.servicesControllerStarted' '.servicesDeployerStarted' '.servicesLogServerStarted' '.servicesSchedulerStarted');
declare -a SCRIPT_NAMES=('rmi.sh' 'os.controller.service.sh' 'os.deployment.service.sh' 'os.log.service.sh' 'os.scheduler.service.sh');
declare -a POLICY_NAMES=('' 'os.controller.service.policy' 'os.deployment.service.policy' 'os.log.service.policy' 'os.scheduler.service.policy');
declare -a PROPERTIES_NAMES=('' 'os.controller.service.properties' 'os.deployment.service.properties' 'os.log.service.properties' 'os.scheduler.service.properties');
declare -a LOG_NAMES=('' 'DeploymentController.log' 'DeployService.log' 'LogServer.log' 'Scheduler.log');
declare -a BOOT_LOG_NAMES=('' 'DeploymentController.boot.log' 'DeployService.boot.log' 'LogServer.boot.log' 'Scheduler.boot.log');

declare -a NEEDED_VARIABLES=('JAVA_HOME' 'ANT_HOME' 'OUTSYSTEMS_HOME');

############################
# Init Script Utils
############################

# Source function library.
if [ -f /lib/lsb/init-functions ]; then
        . /lib/lsb/init-functions
else
        . /etc/rc.d/init.d/functions
fi

### Function Fallbacks ###
# If some of the following logging functions is missing,
# this is a RedHat-based distro, which we can expect it provides
# the success and failure functions that we use

if ! type -p log_success_msg &> /dev/null ; then
        function log_success_msg() {
                success && echo
        }
fi

if ! type -p log_failure_msg &> /dev/null ; then
        function log_failure_msg() {
                failure && echo
        }
fi

if ! type -p log_daemon_msg &> /dev/null ; then
        function log_daemon_msg() {
                echo -n -e "$1"
        }
fi

if ! type -p log_warning_msg &> /dev/null ; then
        function log_warning_msg() {
            echo "[WARNING] $1"
        }
fi

if ! type -p log_end_msg &> /dev/null ; then
        function log_end_msg() {
                if [ "$1" = "0" ]; then
                  log_success_msg
                else
                  log_warning_msg
                fi
        }
fi

############################
#      Util functions      #
############################

function normalizeServiceName() {
	echo $1 | awk '{print toupper($0)}'
}

function getServiceNumber() {
	FRIENDLY_SERVICE_NAME=$(normalizeServiceName $1)

	case "$FRIENDLY_SERVICE_NAME" in
	RMI)
		echo 0
		;;
	CONTROLLER)
		echo 1
		;;
	DEPLOYER)
		echo 2
		;;
	LOGSERVER)
		echo 3
		;;
	SCHEDULER)
		echo 4
		;;
	esac
}

function getServiceJavaClass() {
	echo ${CLASSES[$(getServiceNumber $1)]}
}

function printableName() {
	echo ${PRINTABLE_NAMES[$(getServiceNumber $1)]}
}

function getServiceLockName() {
	echo ${LOCK_NAMES[$(getServiceNumber $1)]}
}

function getScriptName() {
	echo ${SCRIPT_NAMES[$(getServiceNumber $1)]}
}

function getServiceLockFileName() {
	echo $OUTSYSTEMS_HOME/$(getServiceLockName $1)
}

function getServicePolicyFileName() {
	echo /etc/outsystems/${POLICY_NAMES[$(getServiceNumber $1)]}
}

function getServicePropertiesFileName() {
	echo /etc/outsystems/${PROPERTIES_NAMES[$(getServiceNumber $1)]}
}

function getServiceLogFileName() {
	echo $OUTSYSTEMS_HOME/logs/${LOG_NAMES[$(getServiceNumber $1)]}
}

function getServiceBootLogFileName() {
	echo $OUTSYSTEMS_HOME/logs/${BOOT_LOG_NAMES[$(getServiceNumber $1)]}
}

function setJavaVendor() {
	JAVA_VENDOR="sun"
	if [ -d "${JAVA_HOME}/jre/bin/jrockit" ]; then
		JAVA_VENDOR="jrockit"
	else
		for jrpath in "${JAVA_HOME}"/jre/lib/*
		do
			if [ -d "${jrpath}/jrockit" ]; then
				JAVA_VENDOR="jrockit"
			fi
		done
	fi
}

function createOrReplaceOsConfig(){
    #Usage sample: createOrReplaceOsConfig "JAVA_HOME" "/usr/bin/java"
    
    WHAT="$1"
    VALUE="$2"
    
    SYSCONFIG_OS="/etc/sysconfig/outsystems"
    
    grep -q '^\s*'$WHAT'=' $SYSCONFIG_OS
	
	if [ $? -eq 0 ]; then
		sed -i 's,^\s*'$WHAT'=[^\n]*,'$WHAT'='$VALUE',' $SYSCONFIG_OS
	else
		echo "$WHAT=$VALUE" >> $SYSCONFIG_OS
	fi
}

function userIsRoot() {
	if [ $(whoami) = "root" ]; then
		echo "true"
	else
		echo "false"
	fi
}

function checkIsRootService() {
	checkIsRoot "This service must be run as root"
}

function checkIsRoot() {
	if [ $(userIsRoot) = "false" ]; then
		echo $1
		exit 1
	fi
}

function checkVariable() {

	VAR_NAME=$1	
		VAR_VALUE=${!VAR_NAME}

		if [ -z $VAR_VALUE ]; then
			echo "ERROR: $VAR_NAME not defined"
			exit 1
		fi

		if [ ! -d $VAR_VALUE ]; then
			echo $VAR_NAME does not exist as a valid directory : $VAR_VALUE
			exit 1
		fi
}

function checkNeededVariables() {
	for VAR_NAME in ${NEEDED_VARIABLES[@]}
	do
		checkVariable $VAR_NAME
	done
}

function antLibDir() {
	if [ -f $ANT_HOME/lib/ant-launcher.jar ]; then
		echo $ANT_HOME/lib
	else
		echo /usr/share/java
	fi
}

function runService() {
	if [ "$(normalizeServiceName $1)" = "RMI" ]; then
		IS_RMI=true
	fi

	if [ "$IS_RMI" = "" -a -r "$CUSTOM_SH" ]; then
		. $CUSTOM_SH
	fi

	LOCKNAME=$(getServiceLockFileName $1)
	POLICY_FILENAME=$(getServicePolicyFileName $1)
	PROPERTIES_FILENAME=$(getServicePropertiesFileName $1)
	CLASS_NAME=$(getServiceJavaClass $1)
	LOG_FILENAME=$(getServiceLogFileName $1)
	
	# auxiliar variable
	FIRST_RUN="TRUE"
	
	while [ -r $LOCKNAME ]
	do
		# if first time do not sleep
		if [ "$FIRST_RUN" != "TRUE"  ];  then
			sleep 5
		fi
		
		if [ "$IS_RMI" = "" ]; then		
			SERVICE_BOOT_LOG_NAME=$(getServiceBootLogFileName $1)
			
			# Create a empty boot log file (or empty it)...
			echo -n > $SERVICE_BOOT_LOG_NAME

			local MYCOMMAND="LANG=en_US.UTF-8 $JAVA_HOME/bin/java -classpath $CLASSPATH $JAVA_OPTS -Djava.security.policy=$POLICY_FILENAME -Dlog4j.configuration=\"file:$PROPERTIES_FILENAME\" $CLASS_NAME $SERVICE_BOOT_LOG_NAME 2>> $LOG_FILENAME"
			
			# if for some reason we get to this point running as root, lower the privileges!
			if [ $(userIsRoot) = "true" ]; then
				# no need to set umask here, it gets loaded from outsystems .bash_profile
				runCommandAsUser "outsystems" "$MYCOMMAND"
			else
				# making sure outsystem services create files (e.g. deployment files) with appropriate privileges allow application server to read them
				umask 0027
				eval "$MYCOMMAND"
			fi
		else
			# make sure no outsystems java process is runnning (with the old rmi process)
			pkill -u outsystems java

			local MYCOMMAND="$JAVA_HOME/bin/rmiregistry $PORT"
			# if for some reason we get to this point running as root, lower the privileges!
			if [ $(userIsRoot) = "true" ]; then
				runCommandAsUser "outsystems" "$MYCOMMAND"
			else
				eval "$MYCOMMAND"
			fi
		fi
		
		if [ $? -eq 100 ]; then
			rm -f $LOCKNAME
			exit 100
		fi
		
		FIRST_RUN="FALSE"
	done
}

function waitForServiceInitialization() {
	if [ "$(normalizeServiceName $1)" = "RMI" ]; then
		return 0
	fi
	
	SERVICE_LOCK_NAME=$(getServiceLockFileName $1)
	SERVICE_BOOT_LOG_NAME=$(getServiceBootLogFileName $1)
	SLEEP_TIME=3
	MAX_WAIT_COUNT=20
	WAIT_COUNT=0
	
	while [ -e "$SERVICE_LOCK_NAME" -a -e "$SERVICE_BOOT_LOG_NAME" -a "$WAIT_COUNT" -le "$MAX_WAIT_COUNT" ]
	do
		sleep $SLEEP_TIME
		(( WAIT_COUNT = $WAIT_COUNT +1 ))
	done
	
	if [ -e $SERVICE_BOOT_LOG_NAME ]; then		
		# Write a new line to separate from the Starting <service-name> line...
		echo 1>&2
		
		if [ -s $SERVICE_BOOT_LOG_NAME ]; then
			cat $SERVICE_BOOT_LOG_NAME 1>&2
		else 
			echo -n "Timeout while waiting for Service to Start" 1>&2
		fi
		
		return 1
	fi
	
	return 0
}

function askOrDefault(){
    #Usage: askOrDefault "Question to make" "Default"
    #       ANSWER variable, will contain the answer...
    ANSWER=""
    
    while [ -z $ANSWER ]    #This will ensure that no empty answers are returned (if an empty default is provided)    
    do
        printAndLog "$1"
                
        if [ -z "$2" ]     # Fill with default if present
        then
            printAndLog ": "
        else
            printAndLog " [\e[49;32;3m$2\e[m]: "  #Same colors as in configurationtool                    
        fi
        
        read ANSWER
        if [ -z $ANSWER ]
        then
            ANSWER=$2
        fi
    done
}

function askYesNo() {
        local answerYN="$1"

        echo -e -n "${*:2} [\e[49;32;3m$1\e[m] "

        if [ "$silentmode" != "y" ]; then
                read answerYN
                while [ "$answerYN" != "y" -a "$answerYN" != "n" -a "$answerYN" != "" ] ; do
                        echo "Invalid value. Please write 'y' or 'n'"
                        echo -e -n "${*:2} [\e[49;32;3m$1\e[m] "
                        read answerYN
                done
                if [ "$answerYN" = "" ] ; then
                        answerYN="$1"
                fi
        fi

        [ "$answerYN" = "y" ]
}

function stopAndWait() {
    #Usage: stopAndWait getPIDfunction

    PIDS_TO_KILL=$($1)

    if [ "$($1)" != "" ]
    then
        kill -SIGTERM $PIDS_TO_KILL

        #We'll wait up to MAX_WAIT seconds, after that we kill it
        MAX_WAIT=30

        while [ "$($1)" != "" ]
        do
            sleep 1
            if [ $MAX_WAIT -le 0 ]
            then
                return 1
            fi

            (( MAX_WAIT = $MAX_WAIT -1 ))
        done
    fi

    return 0
}

function changeUserUmask() {
	eval UMASK_DIR="$(printf "~%q" "$1")"

	if [ ! -f $UMASK_DIR/.bash_profile ]; then
		cp /etc/skel/.bash_profile $UMASK_DIR/
		chown "$1":"$1" $UMASK_DIR/.bash_profile
	fi
	
	if grep -q 'umask' $UMASK_DIR/.bash_profile; then
		if grep -q 'umask 0027' $UMASK_DIR/.bash_profile; then
			return
		else
			log_warning_msg "'$1' user default umask changed to 0027"
		fi
	fi
	eval echo "umask 0027" >> $UMASK_DIR/.bash_profile
}

# Executes given command as a given user, by using "su - " construct or equivalent
# Usage: runCommandAsUser "USER" "COMMAND AND PARAMETERS"
function runCommandAsUser() {
	
	local EXEC_COMMAND=""
	
	if [ -z "$SUCOMMAND" ]; then
	
		# For SELinux we need to use 'runuser' not 'su'
		if [ -x /sbin/runuser ]
		then
			SUCOMMAND="/sbin/runuser - USER -s /bin/bash -c \"COMMAND\""
		else
			SUCOMMAND="/bin/su - USER -s /bin/bash -c \"COMMAND\""
		fi
	fi
	
	EXEC_COMMAND=${SUCOMMAND//USER/$1}		# Replace USER token with first parameter
	EXEC_COMMAND=${EXEC_COMMAND//COMMAND/$2}	# Replace COMMAND token with second parameter
	
	eval "$EXEC_COMMAND"
}

source /etc/sysconfig/outsystems

setJavaVendor
checkNeededVariables

SERVICESCONFIG=/etc/outsystems/os.services.conf

# Common libs used by PlatformUtil and/or services
COMMON_LIBS="$OUTSYSTEMS_HOME/lib/*"
AVAILABLE_CONNECTORS_DIR="$OUTSYSTEMS_HOME/connectors"
ACTIVE_CONNECTORS_DIR="$OUTSYSTEMS_HOME/lib/active.connectors"
COMMON_CLASSPATH="$COMMON_LIBS:$OUTSYSTEMS_HOME/outsystems.hubedition.runtimeplatformutil.jar:$OUTSYSTEMS_HOME/outsystems.hubedition.runtimecommon.jar:$OUTSYSTEMS_HOME/outsystems.hubedition.common.jar:$OUTSYSTEMS_HOME/outsystems.hubedition.runtimeplatform.jar:$OUTSYSTEMS_HOME/outsystems.hubedition.servercommon.jar:$OUTSYSTEMS_HOME/outsystems.runtimeservices.jar:$ACTIVE_CONNECTORS_DIR/*"

if [ -f $ANT_HOME/lib/ant-launcher.jar ]; then
	ANT_LIBS=$ANT_HOME/lib
else
	ANT_LIBS=/usr/share/java
fi
